@@ -522,6 +522,7 @@ class Say(Node):
         'with_',
         'interact',
         'attributes',
+        'arguments',
         ]
 
     def diff_info(self):
@@ -531,9 +532,10 @@ class Say(Node):
         self = Node.__new__(cls)
         self.attributes = None
         self.interact = True
+        self.arguments = None
         return self
 
-    def __init__(self, loc, who, what, with_, interact=True, attributes=None):
+    def __init__(self, loc, who, what, with_, interact=True, attributes=None, arguments=None):
 
         super(Say, self).__init__(loc)
 
@@ -552,6 +554,7 @@ class Say(Node):
         self.what = what
         self.with_ = with_
         self.interact = interact
+        self.arguments = arguments
 
         # A tuple of attributes that are applied to the character that's
         # speaking, or None to disable this behavior.
@@ -605,12 +608,20 @@ class Say(Node):
 
             renpy.store._last_raw_what = what
 
+            if self.arguments is not None:
+                args, kwargs = self.arguments.evaluate()
+            else:
+                args = tuple()
+                kwargs = dict()
+
             if getattr(who, "record_say", True):
                 renpy.store._last_say_who = self.who
                 renpy.store._last_say_what = what
+                renpy.store._last_say_args = args
+                renpy.store._last_say_kwargs = kwargs
 
             say_menu_with(self.with_, renpy.game.interface.set_transition)
-            renpy.exports.say(who, what, interact=self.interact)
+            renpy.exports.say(who, what, interact=self.interact, *args, **kwargs)
 
         finally:
             renpy.game.context().say_attributes = None
@@ -659,6 +670,7 @@ class Say(Node):
 
         return rv
 
+
 # Copy the descriptor.
 setattr(Say, "with", Say.with_)  # E1101
 
@@ -695,6 +707,7 @@ class Init(Node):
 
     def execute(self):
         next_node(self.next)
+        renpy.execution.not_infinite_loop(60)
         statement_name("init")
 
     def restructure(self, callback):
@@ -856,6 +869,7 @@ class EarlyPython(Node):
 
     def execute(self):
         next_node(self.next)
+        renpy.execution.not_infinite_loop(60)
         statement_name("python early")
 
     def early_execute(self):
@@ -1373,6 +1387,16 @@ class Return(Node):
         else:
             renpy.store._return = None
 
+        ctx = renpy.game.context()
+
+        if renpy.game.context().init_phase:
+            if len(ctx.return_stack) == 0:
+
+                if renpy.config.developer:
+                    raise Exception("Unexpected return during the init phase.")
+
+                return
+
         next_node(renpy.game.context().lookup_return(pop=True))
         renpy.game.context().pop_dynamic()
 
@@ -1490,6 +1514,7 @@ class Menu(Node):
             if block is not None:
                 callback(block)
 
+
 setattr(Menu, "with", Menu.with_)  # E1101
 
 
@@ -1678,11 +1703,14 @@ class UserStatement(Node):
         'line',
         'parsed',
         'block',
-        'translatable' ]
+        'translatable',
+        'code_block',
+        ]
 
     def __new__(cls, *args, **kwargs):
         self = Node.__new__(cls)
         self.block = [ ]
+        self.code_block = None
         self.translatable = False
         return self
 
@@ -1691,14 +1719,55 @@ class UserStatement(Node):
         super(UserStatement, self).__init__(loc)
         self.line = line
         self.block = block
+        self.code_block = None
         self.parsed = None
 
         # Do not store the parse quite yet.
         _parse_info = renpy.statements.parse(self, self.line, self.block)
 
+    def get_children(self, f):
+        f(self)
+
+        if not self.code_block:
+            return
+
+        for i in self.code_block:
+            i.get_children(f)
+
+    def chain(self, next):  # @ReservedAssignment
+        self.next = next
+
+        if self.code_block is not None:
+            chain_block(self.code_block, next)
+
+    def replace_next(self, old, new):
+        Node.replace_next(self, old, new)
+
+        if (self.code_block) and (self.code_block[0] is old):
+            self.code_block.insert(0, new)
+
+    def restructure(self, callback):
+        if self.code_block:
+            callback(self.code_block)
+
     def diff_info(self):
         return (UserStatement, self.line)
 
+    def call(self, method, *args, **kwargs):
+
+        parsed = self.parsed
+        if parsed is None:
+            parsed = renpy.statements.parse(self, self.line, self.block)
+            self.parsed = parsed
+
+        return renpy.statements.call(method, parsed, *args, **kwargs)
+
+    def execute_init(self):
+        self.call("execute_init")
+
+    def get_init(self):
+        return 0, self.execute_init
+
     def execute(self):
         next_node(self.get_next())
         statement_name(self.get_name())
@@ -1714,15 +1783,6 @@ class UserStatement(Node):
 
         return [ self.get_next() ]
 
-    def call(self, method, *args, **kwargs):
-
-        parsed = self.parsed
-        if parsed is None:
-            parsed = renpy.statements.parse(self, self.line, self.block)
-            self.parsed = parsed
-
-        return renpy.statements.call(method, parsed, *args, **kwargs)
-
     def get_name(self):
         parsed = self.parsed
         if parsed is None:
@@ -1732,7 +1792,12 @@ class UserStatement(Node):
         return renpy.statements.get_name(parsed)
 
     def get_next(self):
-        rv = self.call("next")
+
+        if self.code_block and len(self.code_block):
+            rv = self.call("next", self.code_block[0].name)
+        else:
+            rv = self.call("next")
+
         if rv is not None:
             return renpy.game.script.lookup(rv)
         else:
@@ -1773,6 +1838,7 @@ def get_namespace(store):
 
     return StoreNamespace(store), False
 
+
 # Config variables that are set twice - once when the rpy is first loaded,
 # and then again at init time.
 EARLY_CONFIG = { "save_directory" }
@@ -1880,13 +1946,19 @@ class Default(Node):
         d = renpy.python.store_dicts[self.store]
 
         defaults_set = d.get("_defaults_set", None)
+
         if defaults_set is None:
             d["_defaults_set"] = defaults_set = renpy.python.RevertableSet()
+            d.ever_been_changed.add("_defaults_set")
 
         if self.varname not in defaults_set:
             d[self.varname] = renpy.python.py_eval_bytecode(self.code.bytecode)
+            d.ever_been_changed.add(self.varname)
+
             defaults_set.add(self.varname)
+
         else:
+
             if start and renpy.config.developer:
                 raise Exception("{}.{} is being given a default a second time.".format(self.store, self.varname))
 
