@@ -1,4 +1,4 @@
-# Copyright 2004-2015 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -47,9 +47,10 @@ def get_available_image_tags():
 
     return [ k for k, v in image_attributes.items() if v ]
 
+
 def get_available_image_attributes(tag, attributes=()):
     """
-    :doc: image_func
+    :undocumented:
 
     Returns a list of tuples, with each tuple representing a possible
     combination of image attributes that can be associated with `tag`.
@@ -72,6 +73,94 @@ def get_available_image_attributes(tag, attributes=()):
     return rv
 
 
+def get_tag_method(tag, method):
+    """
+    This looks for an image named `tag`, then tries to grab the given method
+    from it. If it can do that, returns a reference to the method. Otherwise,
+    it returns None.
+    """
+
+    ti =  images.get((tag,), None)
+
+    if ti is None:
+        return None
+
+    return getattr(ti, method, None)
+
+
+def check_image_attributes(tag, attributes):
+    """
+    :doc: image_func
+
+    Checks to see if there is a unique image with the given tag and
+    attributes. If there is, returns the attributes in order.
+    Otherwise, returns None.
+    """
+
+    ca = get_tag_method(tag, "_choose_attributes")
+
+    if ca is not None:
+        return ca(tag, attributes, None)
+
+    l = get_available_image_attributes(tag, attributes)
+
+    # Check to see if there's an image that is exactly the one we want.
+    for i in l:
+        if len(i) == len(attributes):
+            return tuple(i)
+
+    if len(l) != 1:
+        return None
+
+    return tuple(l[0])
+
+
+def get_ordered_image_attributes(tag, attributes=(), sort=None):
+    """
+    :doc: image_func
+
+    Returns a list of image tags, ordered in a way that makes sense to
+    present to the user.
+
+    `attributes`
+        If present, only attributes that are compatible with the given
+        attributes are considered. (Compatible means that the attributes
+        can be in a single image at the same time.)
+
+    `sort`
+        If not None, the returned list of attributes is sorted. This is a function
+        that should be used as a tiebreaker.
+
+    """
+
+    la = get_tag_method(tag, "_list_attributes")
+
+    if la is not None:
+        return la(tag, attributes)
+
+    attrcount = collections.defaultdict(int)
+    attrtotalpos = collections.defaultdict(float)
+
+    for attrlist in get_available_image_attributes(tag, attributes):
+        for i, attr in enumerate(attrlist):
+            attrcount[attr] += 1
+            attrtotalpos[attr] += i
+
+    if sort is None:
+
+        return list(attrcount.keys())
+
+    else:
+
+        l = [ ]
+
+        for attr in attrcount:
+            l.append((attrtotalpos[attr] / attrcount[attr], sort(attr), attr))
+
+        l.sort()
+        return [ i[2] for i in l ]
+
+
 def register_image(name, d):
     """
     Registers the existence of an image with `name`, and that the image
@@ -135,9 +224,18 @@ class ImageReference(renpy.display.core.Displayable):
     """
 
     nosave = [ 'target' ]
+
     target = None
+    old_transform = None
     param_target = None
 
+    __version__ = 1
+
+    def after_upgrade(self, version):
+        if version < 1:
+            if isinstance(self.param_target, renpy.display.transform.Transform):
+                self.old_transform = self.param_target
+
     def __init__(self, name, **properties):
         """
         @param name: A tuple of strings, the name of the image. Or else
@@ -166,18 +264,15 @@ class ImageReference(renpy.display.core.Displayable):
 
         return True
 
-    def _get_parameterized(self):
-        if self.param_target:
-            return self.param_target._get_parameterized()
+    def _target(self):
 
-        return self
+        if self.target is None:
+            self.find_target()
+
+        return self.target._target()
 
     def find_target(self):
 
-        if self.param_target:
-            self.target = self.param_target
-            return None
-
         name = self.name
 
         if isinstance(name, renpy.display.core.Displayable):
@@ -187,81 +282,117 @@ class ImageReference(renpy.display.core.Displayable):
         if not isinstance(name, tuple):
             name = tuple(name.split())
 
-        parameters = [ ]
-
         def error(msg):
             self.target = renpy.text.text.Text(msg, color=(255, 0, 0, 255), xanchor=0, xpos=0, yanchor=0, ypos=0)
 
             if renpy.config.debug:
                 raise Exception(msg)
 
+        args = [ ]
 
-        # Scan through, searching for an image (defined with an
-        # input statement) that is a prefix of the given name.
         while name:
-            if name in images:
-                target = images[name]
+            target = images.get(name, None)
 
-                try:
-                    self.target = target.parameterize(name, parameters)
+            if target is not None:
+                break
 
-                    if self.target is not target:
+            args.insert(0, name[-1])
+            name = name[:-1]
 
-                        if self.param_target is not None:
-                            npt = self.target._get_parameterized()
-                            if isinstance(npt, renpy.display.motion.Transform):
-                                opt = self.param_target._get_parameterized()
-                                npt.take_state(opt)
+        if not name:
+            error("Image '%s' not found." % ' '.join(self.name))
+            return False
 
-                        self.param_target = self.target
+        try:
 
-                except Exception, e:
-                    if renpy.config.debug:
-                        raise
+            a = self._args.copy(name=name, args=args)
 
-                    error(str(e))
+            self.target = target._duplicate(a)
 
-                return True
+        except Exception as e:
 
-            else:
-                parameters.insert(0, name[-1])
-                name = name[:-1]
+            if renpy.config.debug:
+                raise
 
-        error("Image '%s' not found." % ' '.join(self.name))
-        return False
+            error(str(e))
 
-    def parameterize(self, name, parameters):
-        if not self.target:
+        # Copy the old transform over.
+        new_transform = self.target._target()
+
+        if isinstance(new_transform, renpy.display.transform.Transform):
+            if self.old_transform is not None:
+                new_transform.take_state(self.old_transform)
+
+            self.old_transform = new_transform
+
+        else:
+            self.old_transform = None
+
+        return True
+
+    _duplicatable = True
+
+    def _duplicate(self, args):
+
+        rv = self._copy(args)
+        rv.target = None
+
+        if isinstance(rv.name, renpy.display.core.Displayable):
+            if rv.name._duplicatable:
+                rv.name = rv.name._duplicate(args)
+
+        rv.find_target()
+        rv._duplicatable = rv.target._duplicatable
+
+        return rv
+
+    def _unique(self):
+
+        if self.target is None:
             self.find_target()
 
-        return self.target.parameterize(name, parameters)
+        self._duplicatable = self.target._duplicatable
+
+    def _in_current_store(self):
+
+        if self.target is None:
+            self.find_target()
+
+        target = self.target._in_current_store()
+
+        if target is self.target:
+            return self
+
+        rv = self._copy()
+        rv.target = target
+        return rv
 
     def _hide(self, st, at, kind):
-        if not self.target:
+        if self.target is None:
             self.find_target()
 
         return self.target._hide(st, at, kind)
 
     def set_transform_event(self, event):
-        if not self.target:
+        if self.target is None:
             self.find_target()
 
         return self.target.set_transform_event(event)
 
     def event(self, ev, x, y, st):
-        if not self.target:
+        if self.target is None:
             self.find_target()
 
         return self.target.event(ev, x, y, st)
 
     def render(self, width, height, st, at):
-        if not self.target:
+        if self.target is None:
             self.find_target()
 
         return wrap_render(self.target, width, height, st, at)
 
     def get_placement(self):
-        if not self.target:
+        if self.target is None:
             self.find_target()
 
         if not renpy.config.imagereference_respects_position:
@@ -284,7 +415,7 @@ class ImageReference(renpy.display.core.Displayable):
         return xpos, ypos, xanchor, yanchor, xoffset, yoffset, subpixel
 
     def visit(self):
-        if not self.target:
+        if self.target is None:
             self.find_target()
 
         return [ self.target ]
@@ -297,7 +428,7 @@ class DynamicImage(renpy.display.core.Displayable):
 
     A DynamicImage is a displayable that has text interpolation performed
     on it to yield a string giving a new displayable. Such interpolation is
-    performed at the start of each iteraction.
+    performed at the start of each interaction.
     """
 
     nosave = [ 'raw_target' ]
@@ -308,6 +439,9 @@ class DynamicImage(renpy.display.core.Displayable):
     # The raw target that the image resolves to, before it has been parameterized.
     raw_target = None
 
+    # Have we been locked, so we never change?
+    locked = False
+
     def __init__(self, name, scope=None, **properties):
         super(DynamicImage, self).__init__(**properties)
 
@@ -319,6 +453,14 @@ class DynamicImage(renpy.display.core.Displayable):
         else:
             self._uses_scope = False
 
+        if isinstance(name, basestring) and ("[prefix_" in name):
+            self._duplicatable = True
+
+        if isinstance(name, list):
+            for i in name:
+                if ("[prefix_" in i):
+                    self._duplicatable = True
+
     def _scope(self, scope, update):
         return self.find_target(scope, update)
 
@@ -340,32 +482,58 @@ class DynamicImage(renpy.display.core.Displayable):
 
         return True
 
-    def _get_parameterized(self):
+    def _target(self):
         if self.target:
-            return self.target._get_parameterized()
+            return self.target._target()
         else:
             return self
 
     def find_target(self, scope=None, update=True):
 
+        if self.locked and (self.target is not None):
+            return
+
+        if self._args.prefix is None:
+            prefix = ""
+        else:
+            prefix = self._args.prefix
+
         try:
-            target = renpy.easy.dynamic_image(self.name, scope)
+            search = [ ]
+            target = renpy.easy.dynamic_image(self.name, scope, prefix=prefix, search=search)
+        except KeyError as ke:
+            raise Exception("In DynamicImage %r: Could not find substitution '%s'." % (self.name, unicode(ke.args[0])))
         except Exception as e:
             raise Exception("In DynamicImage %r: %r" % (self.name, e))
 
+        if target is None:
+            error = "DynamicImage %r: could not find image." % (self.name,)
+
+            if len(search) == 1:
+                error += " (%r)" % (search[0],)
+            elif len(search) == 2:
+                error += " (%r, %r)" % (search[0], search[1])
+            elif len(search) > 2:
+                error += " (%r, %r, and %d more.)" % (search[0], search[1], len(search) - 2)
+
+            raise Exception(error)
+
         if self.raw_target == target:
             return False
 
         if not update:
             return True
 
-        renpy.display.render.redraw(self, 0)
         self.raw_target = target
-
-        target = target.parameterize('displayable', ())
         old_target = self.target
+
+        if target._duplicatable:
+            target = target._duplicate(self._args)
+
         self.target = target
 
+        renpy.display.render.redraw(self, 0)
+
         if not old_target:
             return True
 
@@ -379,38 +547,55 @@ class DynamicImage(renpy.display.core.Displayable):
 
         return True
 
+    def _duplicate(self, args):
+        rv = self._copy(args)
+        rv.target = None
+        # This does not set _duplicatable, since it should always remain the
+        # same.
+        return rv
+
+    def _in_current_store(self):
+        rv = self._copy()
+
+        if rv.target:
+            rv.target = rv.target._in_current_store()
+
+        rv.locked = True
+        return rv
+
     def _hide(self, st, at, kind):
-        if not self.target:
+        if self.target is None:
             self.find_target()
 
         return self.target._hide(st, at, kind)
 
     def set_transform_event(self, event):
-        if not self.target:
+        if self.target is None:
             self.find_target()
 
         return self.target.set_transform_event(event)
 
     def event(self, ev, x, y, st):
-        if not self.target:
+        if self.target is None:
             self.find_target()
 
         return self.target.event(ev, x, y, st)
 
     def render(self, width, height, st, at):
-        if not self.target:
+        if self.target is None:
             self.find_target()
 
         return wrap_render(self.target, width, height, st, at)
 
     def get_placement(self):
-        if not self.target:
+        if self.target is None:
             self.find_target()
 
         return self.target.get_placement()
 
     def visit(self):
-        if not self.target:
+
+        if self.target is None:
             self.find_target()
 
         return [ self.target ]
@@ -456,7 +641,6 @@ class ShownImageInfo(renpy.object.Object):
             self.attributes = old.attributes.copy()
             self.shown = old.shown.copy()
 
-
     def after_upgrade(self, version):
         if version < 2:
 
@@ -468,15 +652,15 @@ class ShownImageInfo(renpy.object.Object):
                     self.attributes[layer, tag] = self.images[layer][tag][1:]
                     self.shown.add((layer, tag))
 
-    def get_attributes(self, layer, tag):
+    def get_attributes(self, layer, tag, default=()):
         """
         Get the attributes associated the image with tag on the given
         layer.
         """
 
-        return self.attributes.get((layer, tag), ())
+        return self.attributes.get((layer, tag), default)
 
-    def showing(self, layer, name):
+    def showing(self, layer, name, exact=False):
         """
         Returns true if name is the prefix of an image that is showing
         on layer, or false otherwise.
@@ -493,6 +677,9 @@ class ShownImageInfo(renpy.object.Object):
         if len(shown) < len(rest):
             return False
 
+        if exact and (len(shown) != len(rest)):
+            return False
+
         for a, b in zip(shown, rest):
             if a != b:
                 return False
@@ -545,8 +732,7 @@ class ShownImageInfo(renpy.object.Object):
 
         self.shown.discard((layer, tag))
 
-
-    def apply_attributes(self, layer, tag, name):
+    def apply_attributes(self, layer, tag, name, wanted=[], remove=[]):
         """
         Given a layer, tag, and an image name (with attributes),
         returns the canonical name of an image, if one exists. Raises
@@ -555,7 +741,7 @@ class ShownImageInfo(renpy.object.Object):
         """
 
         # If the name matches one that exactly exists, return it.
-        if name in images:
+        if (name in images) and not (wanted or remove):
             return name
 
         nametag = name[0]
@@ -564,7 +750,7 @@ class ShownImageInfo(renpy.object.Object):
         required = set(name[1:])
 
         # The set of attributes a matching image may have.
-        optional = set(self.attributes.get((layer, tag), [ ]))
+        optional = set(wanted) | set(self.attributes.get((layer, tag), [ ]))
 
         # Deal with banned attributes..
         for i in name[1:]:
@@ -572,12 +758,22 @@ class ShownImageInfo(renpy.object.Object):
                 optional.discard(i[1:])
                 required.discard(i)
 
+        for i in remove:
+            optional.discard(i)
+
         return self.choose_image(nametag, required, optional, name)
 
     def choose_image(self, tag, required, optional, exception_name):
 
+        ca = get_tag_method(tag, "_choose_attributes")
+        if ca is not None:
+            attrs = ca(tag, required, optional)
+
+            if attrs is not None:
+                return (tag,) + attrs
+
         # The longest length of an image that matches.
-        max_len = 0
+        max_len = -1
 
         # The list of matching images.
         matches = None
@@ -624,6 +820,7 @@ class ShownImageInfo(renpy.object.Object):
         else:
             return None
 
+
 renpy.display.core.ImagePredictInfo = ShownImageInfo
 
 
@@ -633,4 +830,3 @@ Image = renpy.display.im.image
 Solid = renpy.display.imagelike.Solid
 Frame = renpy.display.imagelike.Frame
 ImageButton = renpy.display.behavior.ImageButton
-
