@@ -1,4 +1,4 @@
-# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -24,6 +24,8 @@
 # their behavior, while functions imported in are probably best left
 # alone as part of the api.
 
+from __future__ import print_function
+
 # Remember the real file.
 _file = file
 
@@ -34,6 +36,7 @@ import renpy.audio
 
 from renpy.pyanalysis import const, pure, not_const
 
+
 def renpy_pure(fn):
     """
     Marks renpy.`fn` as a pure function.
@@ -48,7 +51,8 @@ def renpy_pure(fn):
 
     return fn
 
-from renpy.text.extras import ParameterizedText
+
+from renpy.text.extras import ParameterizedText, filter_text_tags
 from renpy.text.font import register_sfont, register_mudgefont, register_bmfont
 from renpy.text.text import language_tailor
 from renpy.display.behavior import Keymap
@@ -65,7 +69,7 @@ from renpy.display.focus import focus_coordinates
 from renpy.display.predict import screen as predict_screen
 
 from renpy.display.image import image_exists, image_exists as has_image
-from renpy.display.image import get_available_image_tags, get_available_image_attributes
+from renpy.display.image import get_available_image_tags, get_available_image_attributes, check_image_attributes, get_ordered_image_attributes
 
 from renpy.display.im import load_surface, load_image
 
@@ -102,6 +106,8 @@ from renpy.sl2.slparser import CustomParser as register_sl_statement, register_s
 
 from renpy.ast import eval_who
 
+from renpy.loader import add_python_directory
+
 renpy_pure("ParameterizedText")
 renpy_pure("Keymap")
 renpy_pure("has_screen")
@@ -111,18 +117,20 @@ renpy_pure("partial")
 renpy_pure("unelide_filename")
 renpy_pure("known_languages")
 renpy_pure("check_text_tags")
+renpy_pure("filter_text_tags")
 
 import time
 import sys
 import threading
 
+
 def public_api():
     """
     :undocumented:
 
     This does nothing, except to make warnings about unused imports go away.
     """
-    ParameterizedText
+    ParameterizedText, filter_text_tags
     register_sfont, register_mudgefont, register_bmfont
     Keymap
     run, run_action, run_unhovered, run_periodic, map_event
@@ -153,7 +161,7 @@ def public_api():
     map_event, queue_event, clear_keymap_cache
     const, pure, not_const
     image_exists, has_image
-    get_available_image_tags, get_available_image_attributes
+    get_available_image_tags, get_available_image_attributes, check_image_attributes, get_ordered_image_attributes
     load_image, load_surface
     profile_memory, diff_memory, profile_rollback
     TEXT_TAG
@@ -164,6 +172,8 @@ def public_api():
     register_sl_statement, register_sl_displayable
     eval_who
     is_selected, is_sensitive
+    add_python_directory
+
 
 del public_api
 
@@ -303,6 +313,7 @@ def retain_after_load():
 
 scene_lists = renpy.display.core.scene_lists
 
+
 def count_displayables_in_layer(layer):
     """
     Returns how many displayables are in the supplied layer.
@@ -461,6 +472,7 @@ def showing(name, layer='master'):
 
     return renpy.game.context().images.showing(layer, name)
 
+
 def get_showing_tags(layer='master'):
     """
     :doc: image_func
@@ -470,6 +482,22 @@ def get_showing_tags(layer='master'):
 
     return renpy.game.context().images.get_showing_tags(layer)
 
+
+def get_attributes(tag, layer=None):
+    """
+    :doc: image_func
+
+    Return a tuple giving the image attributes for the image with `tag`. If
+    the image is now showing, returns None.
+
+    `layer`
+        The layer to check. If None, uses the default layer for `tag`.
+    """
+
+    layer = default_layer(layer, tag)
+    return renpy.game.context().images.get_attributes(layer, tag, None)
+
+
 def predict_show(name, layer=None, what=None, tag=None, at_list=[ ]):
     """
     :undocumented:
@@ -523,6 +551,8 @@ def predict_show(name, layer=None, what=None, tag=None, at_list=[ ]):
         else:
             img = i(img)
 
+        img._unique()
+
     renpy.game.context().images.predict_show(layer, name, True)
     renpy.display.predict.displayable(img)
 
@@ -635,6 +665,9 @@ def show(name, at_list=[ ], layer=None, what=None, zorder=None, tag=None, behind
         else:
             img = i(img)
 
+        # Mark the newly created images unique.
+        img._unique()
+
     # Update the list of images we have ever seen.
     renpy.game.persistent._seen_images[name] = True  # @UndefinedVariable
 
@@ -710,7 +743,7 @@ def scene(layer='master'):
         renpy.config.missing_scene(layer)
 
 
-def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=None, pixel_width=None): #@ReservedAssignment
+def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=None, pixel_width=None):  # @ReservedAssignment
     """
     :doc: input
 
@@ -750,7 +783,7 @@ def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=N
     if roll_forward is not None:
         default = roll_forward
 
-    fixed = in_fixed_rollback();
+    fixed = in_fixed_rollback()
 
     if has_screen("input"):
         widget_properties = { }
@@ -809,7 +842,6 @@ def menu(items, set_expr):
         def substitute(s):
             return s
 
-
     # Filter the list of items to only include ones for which the
     # condition is true.
     items = [ (substitute(label), value)
@@ -818,12 +850,12 @@ def menu(items, set_expr):
 
     # Filter the list of items on the set_expr:
     if set_expr:
-        set = renpy.python.py_eval(set_expr) #@ReservedAssignment
+        set = renpy.python.py_eval(set_expr)  # @ReservedAssignment
         items = [ (label, value)
                   for label, value in items
                   if label not in set ]
     else:
-        set = None #@ReservedAssignment
+        set = None  # @ReservedAssignment
 
     # Check to see if there's at least one choice in set of items:
     choices = [ value for label, value in items if value is not None ]
@@ -866,7 +898,6 @@ def choice_for_skipping():
         renpy.loadsave.force_autosave(True)
 
 
-
 def predict_menu():
     """
     :undocumented:
@@ -908,7 +939,7 @@ def display_menu(items,
                  scope={ },
                  widget_properties=None,
                  screen="choice",
-                 type="menu", #@ReservedAssignment
+                 type="menu",  # @ReservedAssignment
                  predict_only=False,
                  **kwargs):
     """
@@ -1045,6 +1076,7 @@ def display_menu(items,
 
 
 class TagQuotingDict(object):
+
     def __getitem__(self, key):
 
         store = renpy.store.__dict__
@@ -1061,6 +1093,7 @@ class TagQuotingDict(object):
                 raise Exception("During an interpolation, '%s' was not found as a variable." % key)
             return "<" + key + " unbound>"
 
+
 tag_quoting_dict = TagQuotingDict()
 
 
@@ -1072,7 +1105,7 @@ def predict_say(who, what):
     """
 
     if who is None:
-        who = renpy.store.narrator # E1101 @UndefinedVariable
+        who = renpy.store.narrator  # E1101 @UndefinedVariable
 
     if isinstance(who, (str, unicode)):
         return renpy.store.predict_say(who, what)
@@ -1095,7 +1128,8 @@ def scry_say(who, scry):
     except:
         scry.interacts = True
 
-def say(who, what, interact=True):
+
+def say(who, what, *args, **kwargs):
     """
     :doc: se_say
 
@@ -1113,6 +1147,7 @@ def say(who, what, interact=True):
     `interact`
         If true, Ren'Py waits for player input when displaying the dialogue. If
         false, Ren'Py shows the dialogue, but does not perform an interaction.
+        (This is passed in as a keyword argument.)
 
     This function is rarely necessary, as the following three lines are
     equivalent. ::
@@ -1127,12 +1162,15 @@ def say(who, what, interact=True):
         what = what % tag_quoting_dict
 
     if who is None:
-        who = renpy.store.narrator # E1101 @UndefinedVariable
+        who = renpy.store.narrator  # E1101 @UndefinedVariable
+
+    if renpy.config.say_arguments_callback:
+        args, kwargs = renpy.config.say_arguments_callback(who, *args, **kwargs)
 
     if isinstance(who, (str, unicode)):
-        renpy.store.say(who, what, interact=interact)
+        renpy.store.say(who, what, *args, **kwargs)
     else:
-        who(what, interact=interact)
+        who(what, *args, **kwargs)
 
 
 def imagemap(ground, selected, hotspots, unselected=None, overlays=False,
@@ -1226,7 +1264,6 @@ def pause(delay=None, music=None, with_none=None, hard=False, checkpoint=None):
         a checkpoint will only be set if delay is set.
     """
 
-
     if checkpoint is None:
         if delay is not None:
             checkpoint = False
@@ -1248,7 +1285,7 @@ def pause(delay=None, music=None, with_none=None, hard=False, checkpoint=None):
         if newdelay is not None:
             delay = newdelay
 
-    if renpy.game.after_rollback and roll_forward is None:
+    if (delay is not None) and renpy.game.after_rollback and roll_forward is None:
         delay = 0
 
     if delay is None:
@@ -1322,7 +1359,6 @@ def movie_cutscene(filename, delay=None, loops=0, stop_music=True):
         roll_forward = None
 
     rv = renpy.ui.interact(suppress_overlay=True,
-                           show_mouse=False,
                            roll_forward=roll_forward)
 
     # We don't want to put a checkpoint here, as we can't roll back while
@@ -1369,10 +1405,11 @@ def with_statement(trans, always=False, paired=None, clear=True):
 
     return renpy.game.interface.do_with(trans, paired, clear=clear)
 
+
 globals()["with"] = with_statement
 
 
-def rollback(force=False, checkpoints=1, defer=False, greedy=True, label=None):
+def rollback(force=False, checkpoints=1, defer=False, greedy=True, label=None, abnormal=True):
     """
     :doc: rollback
 
@@ -1388,8 +1425,8 @@ def rollback(force=False, checkpoints=1, defer=False, greedy=True, label=None):
         will roll back as far as it can, subject to this condition.
 
     `defer`
-        If true, the call will be deferred until code from the main context is
-        executed.
+        If true, the call will be deferred until control returns to the main
+        context.
 
     `greedy`
         If true, rollback will finish just after the previous checkpoint.
@@ -1397,6 +1434,11 @@ def rollback(force=False, checkpoints=1, defer=False, greedy=True, label=None):
 
     `label`
         If not None, a label that is called when rollback completes.
+
+    `abnormal`
+        If true, the default, script executed after the transition is run in
+        an abnormal mode that skips transitions that would have otherwise
+        occured. Abnormal mode ends when an interaction begins.
     """
 
     if defer and len(renpy.game.contexts) > 1:
@@ -1416,7 +1458,7 @@ def rollback(force=False, checkpoints=1, defer=False, greedy=True, label=None):
 
     renpy.config.skipping = None
     renpy.game.log.complete()
-    renpy.game.log.rollback(checkpoints, greedy=greedy, label=label, force=(force is True))
+    renpy.game.log.rollback(checkpoints, greedy=greedy, label=label, force=(force is True), abnormal=abnormal)
 
 
 def toggle_fullscreen():
@@ -1434,6 +1476,7 @@ def toggle_music():
     Does nothing.
     """
 
+
 @renpy_pure
 def has_label(name):
     """
@@ -1448,6 +1491,7 @@ def has_label(name):
 
     return renpy.game.script.has_label(name)
 
+
 @renpy_pure
 def get_all_labels():
     """
@@ -1518,20 +1562,25 @@ def reload_script():
     s = get_screen("menu")
 
     if not renpy.store.main_menu:
+
         if s is not None:
             session["_reload_screen"] = s.screen_name[0]
+            session["_reload_screen_args"] = s.scope.get("_args", ())
+            session["_reload_screen_kwargs"] = s.scope.get("_kwargs", { })
 
         renpy.game.call_in_new_context("_save_reload_game")
 
     else:
+
         if s is not None:
             session["_main_menu_screen"] = s.screen_name[0]
+            session["_main_menu_screen_args"] = s.scope.get("_args", ())
+            session["_main_menu_screen_kwargs"] = s.scope.get("_kwargs", { })
 
         utter_restart()
 
 
-
-def quit(relaunch=False, status=0): #@ReservedAssignment
+def quit(relaunch=False, status=0):  # @ReservedAssignment
     """
     :doc: other
 
@@ -1577,9 +1626,17 @@ def call(label, *args, **kwargs):
     Causes the current Ren'Py statement to terminate, and a jump to a
     `label` to occur. When the jump returns, control will be passed
     to the statement following the current statement.
+
+    `from_current`
+        If true, control will return to the current statement, rather than
+        the statement following the current statement. (This will lead to
+        the current statement being run twice. This must be passed as a
+        keyword argument.)
     """
 
-    raise renpy.game.CallException(label, args, kwargs)
+    from_current = kwargs.pop("from_current", False)
+    raise renpy.game.CallException(label, args, kwargs, from_current=from_current)
+
 
 def return_statement():
     """
@@ -1596,12 +1653,16 @@ def screenshot(filename):
     :doc: other
 
     Saves a screenshot in `filename`.
+
+    Returns True if the screenshot was saved successfully, False if saving
+    failed for some reason.
     """
 
-    renpy.game.interface.save_screenshot(filename)
+    return renpy.game.interface.save_screenshot(filename)
+
 
 @renpy_pure
-def version(tuple=False): #@ReservedAssignment
+def version(tuple=False):  # @ReservedAssignment
     """
     :doc: renpy_version
 
@@ -1617,11 +1678,24 @@ def version(tuple=False): #@ReservedAssignment
 
     return renpy.version
 
+
 version_string = renpy.version
 version_only = renpy.version_only
 version_name = renpy.version_name
 version_tuple = renpy.version_tuple
-license = "" # @ReservedAssignment
+license = ""  # @ReservedAssignment
+
+try:
+    import platform as _platform
+    platform = "-".join(_platform.platform().split("-")[:2])
+except:
+    if renpy.android:
+        platform = "Android"
+    elif renpy.ios:
+        platform = "iOS"
+    else:
+        platform = "Unknown"
+
 
 def transition(trans, layer=None, always=False, force=False):
     """
@@ -1681,6 +1755,7 @@ def get_game_runtime():
 
     return renpy.game.contexts[0].runtime
 
+
 @renpy_pure
 def loadable(filename):
     """
@@ -1796,6 +1871,7 @@ def get_filename_line():
 # A file that log logs to.
 logfile = None
 
+
 def log(msg):
     """
     :doc: debug
@@ -1824,7 +1900,7 @@ def log(msg):
 
         import textwrap
 
-        print >>logfile, textwrap.fill(msg, renpy.config.log_width).encode("utf-8")
+        print(textwrap.fill(msg, renpy.config.log_width).encode("utf-8"), file=logfile)
         logfile.flush()
 
     except:
@@ -1842,21 +1918,26 @@ def force_full_redraw():
     renpy.game.interface.full_redraw = True
 
 
-def do_reshow_say(who, what, interact=False):
+def do_reshow_say(who, what, interact=False, *args, **kwargs):
 
     if who is not None:
         who = renpy.python.py_eval(who)
 
-    say(who, what, interact=interact)
+    say(who, what, interact=interact, *args, **kwargs)
+
 
 curried_do_reshow_say = curry(do_reshow_say)
 
 
 def get_reshow_say(**kwargs):
+    kw = dict(renpy.store._last_say_kwargs)
+    kw.update(kwargs)
+
     return curried_do_reshow_say(
         renpy.store._last_say_who,
         renpy.store._last_say_what,
-        **kwargs)
+        renpy.store._last_say_args,
+        **kw)
 
 
 def reshow_say(**kwargs):
@@ -1871,7 +1952,7 @@ def last_interact_type():
     return getattr(renpy.game.context().info, "_last_interact_type", None)
 
 
-def dynamic(*vars): #@ReservedAssignment
+def dynamic(*vars):  # @ReservedAssignment
     """
     :doc: other
 
@@ -1887,7 +1968,7 @@ def dynamic(*vars): #@ReservedAssignment
     renpy.game.context().make_dynamic(vars)
 
 
-def context_dynamic(*vars): #@ReservedAssignment
+def context_dynamic(*vars):  # @ReservedAssignment
     """
     :doc: other
 
@@ -1943,7 +2024,7 @@ def seen_image(name):
     return name in renpy.game.persistent._seen_images  # @UndefinedVariable
 
 
-def file(fn): #@ReservedAssignment
+def file(fn):  # @ReservedAssignment
     """
     :doc: file
 
@@ -1958,6 +2039,15 @@ def file(fn): #@ReservedAssignment
     return renpy.loader.load(fn)
 
 
+def notl_file(fn):  # @ReservedAssignment
+    """
+    :undocumented:
+
+    Like file, but doesn't search the translation prefix.
+    """
+    return renpy.loader.load(fn, tl=False)
+
+
 @renpy_pure
 def image_size(im):
     """
@@ -2002,18 +2092,25 @@ def get_at_list(name, layer=None):
     return renpy.game.context().scene_lists.at_list[layer].get(tag, None)
 
 
-def show_layer_at(at_list, layer='master'):
+def show_layer_at(at_list, layer='master', reset=True):
     """
     :doc: se_images
+    :name: renpy.show_layer_at
 
     The python equivalent of the ``show layer`` `layer` ``at`` `at_list`
     statement.
+
+    `reset`
+        If true, the transform state is reset to the start when it is shown.
+        If false, the transform state is persisted, allowing the new transform
+        to update that state.
     """
 
     if not isinstance(at_list, list):
         at_list = [ at_list ]
 
-    renpy.game.context().scene_lists.set_layer_at_list(layer, at_list)
+    renpy.game.context().scene_lists.set_layer_at_list(layer, at_list, reset=reset)
+
 
 layer_at_list = show_layer_at
 
@@ -2028,6 +2125,7 @@ def free_memory():
     force_full_redraw()
     renpy.display.interface.kill_textures_and_surfaces()
 
+
 @renpy_pure
 def easy_displayable(d, none=False):
     """
@@ -2060,6 +2158,7 @@ def iconify():
 
     renpy.game.interface.iconify()
 
+
 # New context stuff.
 call_in_new_context = renpy.game.call_in_new_context
 curried_call_in_new_context = renpy.curry.curry(renpy.game.call_in_new_context)
@@ -2075,14 +2174,18 @@ renpy_pure("curried_invoke_in_new_context")
 def _error(msg):
     raise Exception(msg)
 
+
 _error_handlers = [ _error ]
 
+
 def push_error_handler(eh):
     _error_handlers.append(eh)
 
+
 def pop_error_handler():
     _error_handlers.pop()
 
+
 def error(msg):
     _error_handlers[-1](msg)
 
@@ -2135,6 +2238,7 @@ def scry():
     node = renpy.game.script.lookup(name)
     return node.scry()
 
+
 @renpy_pure
 def munged_filename():
     return renpy.parser.munge_filename(get_filename_line()[0])
@@ -2144,17 +2248,19 @@ def munged_filename():
 
 loaded_modules = set()
 
+
 def load_module(name, **kwargs):
     """
     :doc: other
 
-    This loads the Ren'Py module named name. A Ren'Py module consists of Ren'Py code
+    This loads the Ren'Py module named name. A Ren'Py module consists of Ren'Py script
     that is loaded into the usual (store) namespace, contained in a file named
     name.rpym or name.rpymc. If a .rpym file exists, and is newer than the
     corresponding .rpymc file, it is loaded and a new .rpymc file is created.
 
-    All init code in the module is run before this function returns. An error is
-    raised if the module name cannot be found, or is ambiguous.
+    All of the init blocks (and other init-phase code) in the module are run
+    before this function returns. An error is raised if the module name cannot
+    be found, or is ambiguous.
 
     Module loading may only occur from inside an init block.
     """
@@ -2177,10 +2283,13 @@ def load_module(name, **kwargs):
     renpy.game.contexts.append(context)
 
     context.make_dynamic(kwargs)
-    renpy.store.__dict__.update(kwargs) #@UndefinedVariable
+    renpy.store.__dict__.update(kwargs)  # @UndefinedVariable
 
-    for prio, node in initcode: #@UnusedVariable
-        renpy.game.context().run(node)
+    for prio, node in initcode:  # @UnusedVariable
+        if isinstance(node, renpy.ast.Node):
+            renpy.game.context().run(node)
+        else:
+            node()
 
     context.pop_all_dynamic()
 
@@ -2217,8 +2326,11 @@ def load_string(s, filename="<string>"):
         context.init_phase = True
         renpy.game.contexts.append(context)
 
-        for prio, node in initcode: #@UnusedVariable
-            renpy.game.context().run(node)
+        for prio, node in initcode:  # @UnusedVariable
+            if isinstance(node, renpy.ast.Node):
+                renpy.game.context().run(node)
+            else:
+                node()
 
         context.pop_all_dynamic()
         renpy.game.contexts.pop()
@@ -2247,6 +2359,7 @@ def pop_call():
 
     renpy.game.context().pop_call()
 
+
 pop_return = pop_call
 
 
@@ -2287,6 +2400,7 @@ def shown_window():
 
 
 class placement(renpy.python.RevertableObject):
+
     def __init__(self, p):
         super(placement, self).__init__()
 
@@ -2298,6 +2412,7 @@ class placement(renpy.python.RevertableObject):
         self.yoffset = p[5]
         self.subpixel = p[6]
 
+
 def get_placement(d):
     """
     :doc: image_func
@@ -2352,14 +2467,17 @@ def get_image_bounds(tag, width=None, height=None, layer='master'):
 
 # User-Defined Displayable stuff.
 
+
 Render = renpy.display.render.Render
 render = renpy.display.render.render
 IgnoreEvent = renpy.display.core.IgnoreEvent
 redraw = renpy.display.render.redraw
 
+
 class Displayable(renpy.display.core.Displayable, renpy.python.RevertableObject):
     pass
 
+
 class Container(renpy.display.core.Displayable, renpy.python.RevertableObject):
     _list_type = renpy.python.RevertableList
 
@@ -2387,7 +2505,6 @@ def cache_pin(*args):
     renpy.store._cache_pin_set = new_pins | renpy.store._cache_pin_set
 
 
-
 def cache_unpin(*args):
     """
     :undocumented: Cache pin is deprecated.
@@ -2443,7 +2560,6 @@ def stop_predict(*args):
 
 
 def start_predict_screen(_screen_name, *args, **kwargs):
-
     """
     :doc: screens
 
@@ -2494,7 +2610,7 @@ def call_screen(_screen_name, *args, **kwargs):
 
     try:
         rv = renpy.ui.interact(mouse="screen", type="screen", roll_forward=roll_forward)
-    except (renpy.game.JumpException, renpy.game.CallException), e:
+    except (renpy.game.JumpException, renpy.game.CallException) as e:
         rv = e
 
     renpy.exports.checkpoint(rv)
@@ -2512,6 +2628,7 @@ def call_screen(_screen_name, *args, **kwargs):
 
     return rv
 
+
 @renpy_pure
 def list_files(common=False):
     """
@@ -2527,12 +2644,14 @@ def list_files(common=False):
 
     rv = [ ]
 
-    for dir, fn in renpy.loader.listdirfiles(common): #@ReservedAssignment
+    for dir, fn in renpy.loader.listdirfiles(common):  # @ReservedAssignment
         if fn.startswith("saves/"):
             continue
 
         rv.append(fn)
 
+    rv.sort()
+
     return rv
 
 
@@ -2555,7 +2674,7 @@ def get_renderer_info():
 
     Other, renderer-specific, keys may also exist. The dictionary should
     be treated as immutable. This should only be called once the display
-    has been started (that is, after the init code is finished).
+    has been started (that is, after the init phase has finished).
     """
 
     return renpy.display.draw.info
@@ -2601,6 +2720,7 @@ def mode(mode):
 
     modes.insert(0, mode)
 
+
 def get_mode():
     """
     :doc: modes
@@ -2617,6 +2737,7 @@ def get_mode():
 
     return modes[0]
 
+
 def notify(message):
     """
     :doc: other
@@ -2670,7 +2791,7 @@ def vibrate(duration):
     """
 
     try:
-        import android #@UnresolvedImport
+        import android  # @UnresolvedImport
         android.vibrate(duration)
     except:
         pass
@@ -2754,6 +2875,8 @@ def set_physical_size(size):
     if get_renderer_info()["resizable"]:
         renpy.display.draw.quit()
         renpy.display.interface.set_mode(size)
+        renpy.display.interface.last_resize = size
+
 
 def reset_physical_size():
     """
@@ -2803,7 +2926,7 @@ def fsdecode(s):
     return s.decode(fsencoding)
 
 
-from renpy.editor import launch_editor #@UnusedImport
+from renpy.editor import launch_editor  # @UnusedImport
 
 
 def get_image_load_log(age=None):
@@ -2832,7 +2955,6 @@ def get_image_load_log(age=None):
     else:
         deadline = 0
 
-
     for i in renpy.display.im.cache.load_log:
         if i[0] < deadline:
             break
@@ -2944,6 +3066,7 @@ def count_seen_dialogue_blocks():
 
     return renpy.game.seen_translates_count
 
+
 def count_newly_seen_dialogue_blocks():
     """
     :doc: other
@@ -2955,7 +3078,6 @@ def count_newly_seen_dialogue_blocks():
     return renpy.game.new_translates_count
 
 
-
 def substitute(s, scope=None, translate=True):
     """
     :doc: other
@@ -3011,6 +3133,7 @@ def get_return_stack():
 
     return renpy.game.context().get_return_stack()
 
+
 def set_return_stack(stack):
     """
     :doc: label
@@ -3024,6 +3147,7 @@ def set_return_stack(stack):
 
     renpy.game.context().set_return_stack(stack)
 
+
 def invoke_in_thread(fn, *args, **kwargs):
     """
     :doc: other
@@ -3049,6 +3173,7 @@ def invoke_in_thread(fn, *args, **kwargs):
     t.daemon = True
     t.start()
 
+
 def cancel_gesture():
     """
     :doc: gesture
@@ -3057,7 +3182,8 @@ def cancel_gesture():
     This should be called by displayables that have gesture-like behavior.
     """
 
-    renpy.display.gesture.recognizer.cancel() # @UndefinedVariable
+    renpy.display.gesture.recognizer.cancel()  # @UndefinedVariable
+
 
 def execute_default_statement(start=False):
     """
@@ -3069,6 +3195,7 @@ def execute_default_statement(start=False):
     for i in renpy.ast.default_statements:
         i.set_default(start)
 
+
 def write_log(s, *args):
     """
     :undocumented:
@@ -3078,6 +3205,7 @@ def write_log(s, *args):
 
     renpy.display.log.write(s, *args)
 
+
 def predicting():
     """
     :doc: screens
@@ -3087,6 +3215,7 @@ def predicting():
 
     return renpy.display.predict.predicting
 
+
 def get_line_log():
     """
     :undocumented:
@@ -3097,6 +3226,7 @@ def get_line_log():
 
     return renpy.game.context().line_log[:]
 
+
 def clear_line_log():
     """
     :undocumented:
@@ -3106,6 +3236,7 @@ def clear_line_log():
 
     renpy.game.context().line_log = [ ]
 
+
 def add_layer(layer, above=None, below=None, menu_clear=True):
     """
     :doc: other
@@ -3157,7 +3288,7 @@ def add_layer(layer, above=None, below=None, menu_clear=True):
     layers.insert(index, layer)
 
     if menu_clear:
-        renpy.config.menu_clear_layers.append(layer) # @UndefinedVariable
+        renpy.config.menu_clear_layers.append(layer)  # @UndefinedVariable
 
 
 def maximum_framerate(t):
@@ -3270,6 +3401,7 @@ def get_identifier_checkpoints(identifier):
 
     return renpy.game.log.get_identifier_checkpoints(identifier)
 
+
 def get_adjustment(bar_value):
     """
     :doc: other
@@ -3287,3 +3419,25 @@ def get_adjustment(bar_value):
     """
 
     return bar_value.get_adjustment()
+
+
+def get_skipping():
+    """
+    :doc: other
+
+    Returns true if the Ren'Py is skipping, "fast" if Ren'Py is fast skipping,
+    and false if it is not skipping.
+    """
+
+    return renpy.config.skipping
+
+
+def get_texture_size():
+    """
+    :undocumented:
+
+    Returns the number of bytes of memory locked up in OpenGL textures and the
+    number of textures that are defined.
+    """
+
+    return renpy.display.draw.get_texture_size()
