@@ -1,4 +1,4 @@
-# Copyright 2004-2015 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,6 +19,7 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+from __future__ import print_function
 import renpy
 import os.path
 from pickle import loads
@@ -27,13 +28,14 @@ import sys
 import types
 import threading
 import zlib
+import re
 
 # Ensure the utf-8 codec is loaded, to prevent recursion when we use it
 # to look up filenames.
 u"".encode("utf-8")
 
 
-################################################################# Physical Paths
+# Physical Paths
 
 def get_path(fn):
     """
@@ -54,14 +56,15 @@ def get_path(fn):
 
     return fn
 
-################################################################## Asset Loading
+# Asset Loading
+
 
 try:
     import android.apk
 
     expansion = os.environ.get("ANDROID_EXPANSION", None)
     if expansion is not None:
-        print "Using expansion file", expansion
+        print("Using expansion file", expansion)
 
         apks = [
             android.apk.APK(apk=expansion, prefix='assets/x-game/'),
@@ -71,7 +74,7 @@ try:
         game_apks = [ apks[0] ]
 
     else:
-        print "Not using expansion file."
+        print("Not using expansion file.")
 
         apks = [
             android.apk.APK(prefix='assets/x-game/'),
@@ -95,6 +98,7 @@ old_config_archives = None
 # A map from lower-case filename to regular-case filename.
 lower_map = { }
 
+
 def index_archives():
     """
     Loads in the indexes for the archive files. Also updates the lower_map.
@@ -163,10 +167,11 @@ def index_archives():
         except:
             raise
 
-    for dir, fn in listdirfiles(): #@ReservedAssignment
+    for dir, fn in listdirfiles():  # @ReservedAssignment
         lower_map[fn.lower()] = fn
 
-def walkdir(dir): #@ReservedAssignment
+
+def walkdir(dir):  # @ReservedAssignment
     rv = [ ]
 
     if not os.path.exists(dir) and not renpy.config.developer:
@@ -191,6 +196,7 @@ game_files = [ ]
 # A list of files that are in the common directory.
 common_files = [ ]
 
+
 def cleardirfiles():
     """
     Clears the lists above when the game has changed.
@@ -202,6 +208,7 @@ def cleardirfiles():
     game_files = [ ]
     common_files = [ ]
 
+
 def scandirfiles():
     """
     Scans directories, archives, and apks and fills out game_files and
@@ -227,9 +234,9 @@ def scandirfiles():
     for apk in apks:
 
         if apk not in game_apks:
-            files = common_files # @UnusedVariable
+            files = common_files  # @UnusedVariable
         else:
-            files = game_files # @UnusedVariable
+            files = game_files  # @UnusedVariable
 
         for f in apk.list():
 
@@ -242,9 +249,9 @@ def scandirfiles():
     for i in renpy.config.searchpath:
 
         if (renpy.config.commondir) and (i == renpy.config.commondir):
-            files = common_files # @UnusedVariable
+            files = common_files  # @UnusedVariable
         else:
-            files = game_files # @UnusedVariable
+            files = game_files  # @UnusedVariable
 
         i = os.path.join(renpy.config.basedir, i)
         for j in walkdir(i):
@@ -257,7 +264,6 @@ def scandirfiles():
             add(None, j)
 
 
-
 def listdirfiles(common=True):
     """
     Returns a list of directory, file tuples known to the system. If
@@ -380,7 +386,7 @@ class SubFile(object):
     def __iter__(self):
         return self
 
-    def next(self): #@ReservedAssignment
+    def next(self):  # @ReservedAssignment
         rv = self.readline()
 
         if not rv:
@@ -425,6 +431,7 @@ class SubFile(object):
     def write(self, s):
         raise Exception("Write not supported by SubFile")
 
+
 open_file = open
 
 if "RENPY_FORCE_SUBFILE" in os.environ:
@@ -437,6 +444,7 @@ if "RENPY_FORCE_SUBFILE" in os.environ:
 
         return SubFile(f, 0, length, '')
 
+
 def load_core(name):
     """
     Returns an open python file object of the given type.
@@ -502,14 +510,18 @@ def load_core(name):
 
     return None
 
-def get_prefixes():
+
+def get_prefixes(tl=True):
     """
     Returns a list of prefixes to search for files.
     """
 
     rv = [ ]
 
-    language = renpy.game.preferences.language
+    if tl:
+        language = renpy.game.preferences.language
+    else:
+        language = None
 
     for prefix in renpy.config.search_prefixes:
 
@@ -520,12 +532,15 @@ def get_prefixes():
 
     return rv
 
-def load(name):
+
+def load(name, tl=True):
 
     if renpy.config.reject_backslash and "\\" in name:
         raise Exception("Backslash in filename, use '/' instead: %r" % name)
 
-    for p in get_prefixes():
+    name = re.sub(r'/+', '/', name).lstrip('/')
+
+    for p in get_prefixes(tl):
         rv = load_core(p + name)
         if rv is not None:
             return rv
@@ -535,6 +550,7 @@ def load(name):
 
 loadable_cache = { }
 
+
 def loadable_core(name):
     """
     Returns True if the name is loadable with load, False if it is not.
@@ -565,8 +581,11 @@ def loadable_core(name):
     loadable_cache[name] = False
     return False
 
+
 def loadable(name):
 
+    name = name.lstrip('/')
+
     for p in get_prefixes():
         if loadable_core(p + name):
             return True
@@ -580,11 +599,13 @@ def transfn(name):
     searched directories.
     """
 
-    name = lower_map.get(name.lower(), name)
+    name = name.lstrip('/')
 
     if renpy.config.reject_backslash and "\\" in name:
         raise Exception("Backslash in filename, use '/' instead: %r" % name)
 
+    name = lower_map.get(name.lower(), name)
+
     if isinstance(name, str):
         name = name.decode("utf-8")
 
@@ -601,6 +622,7 @@ def transfn(name):
 
 hash_cache = dict()
 
+
 def get_hash(name):
     """
     Returns the time the file m was last modified, or 0 if it
@@ -632,7 +654,7 @@ def get_hash(name):
     return rv
 
 
-################################################################# Module Loading
+# Module Loading
 
 class RenpyImporter(object):
     """
@@ -689,24 +711,49 @@ class RenpyImporter(object):
         source = source.encode("raw_unicode_escape")
 
         source = source.replace("\r", "")
-        code = compile(source, filename, 'exec')
+
+        code = compile(source, filename, 'exec', renpy.python.old_compile_flags, 1)
         exec code in mod.__dict__
+
         return mod
 
     def get_data(self, filename):
         return load(filename).read()
 
+
+meta_backup = [ ]
+
+
+def add_python_directory(path):
+    """
+    :doc: other
+
+    Adds `path` to the list of paths searched for Python modules and packages.
+    The path should be a string relative to the game directory. This must be
+    called before an import statement.
+    """
+
+    if path and not path.endswith("/"):
+        path = path + "/"
+
+    sys.meta_path.insert(0, RenpyImporter(path))
+
+
 def init_importer():
-    sys.meta_path.insert(0, RenpyImporter("python-packages/"))
-    sys.meta_path.insert(0, RenpyImporter())
+    meta_backup[:] = sys.meta_path
+
+    add_python_directory("python-packages/")
+    add_python_directory("")
+
 
 def quit_importer():
-    sys.meta_path.pop(0)
-    sys.meta_path.pop(0)
+    sys.meta_path[:] = meta_backup
 
-#################################################################### Auto-Reload
 
-# This is set to True if autoreload hads detected an autoreload is needed.
+# Auto-Reload
+
+
+# This is set to True if autoreload has detected an autoreload is needed.
 needs_autoreload = False
 
 # A map from filename to mtime, or None if the file doesn't exist.
@@ -724,6 +771,7 @@ auto_lock = threading.Condition()
 # Used to indicate that this file is blacklisted.
 auto_blacklisted = renpy.object.Sentinel("auto_blacklisted")
 
+
 def auto_mtime(fn):
     """
     Gets the mtime of fn, or None if the file does not exist.
@@ -734,6 +782,7 @@ def auto_mtime(fn):
     except:
         return None
 
+
 def add_auto(fn, force=False):
     """
     Adds fn as a file we watch for changes. If it's mtime changes or the file
@@ -757,6 +806,7 @@ def add_auto(fn, force=False):
     with auto_lock:
         auto_mtimes[fn] = mtime
 
+
 def auto_thread_function():
     """
     This thread sets need_autoreload when necessary.
@@ -786,6 +836,7 @@ def auto_thread_function():
                     if auto_mtime(fn) != auto_mtimes[fn]:
                         needs_autoreload = True
 
+
 def auto_init():
     """
     Starts the autoreload thread.
@@ -806,6 +857,7 @@ def auto_init():
     auto_thread.daemon = True
     auto_thread.start()
 
+
 def auto_quit():
     """
     Terminates the autoreload thread.
