@@ -30,7 +30,7 @@ registry = { }
 parsers = renpy.parser.ParseTrie()
 
 
-def register(name, parse=None, lint=None, execute=None, predict=None, next=None, scry=None, block=False, init=False, translatable=False):  # @ReservedAssignment
+def register(name, parse=None, lint=None, execute=None, predict=None, next=None, scry=None, block=False, init=False, translatable=False, execute_init=None):  # @ReservedAssignment
     """
     :doc: statement_register
     :name: renpy.register_statement
@@ -42,45 +42,59 @@ def register(name, parse=None, lint=None, execute=None, predict=None, next=None,
         empty string to define a new default statement (the default statement will
         replace the say statement).
 
+    `block`
+        When this is False, the statement does not expect a block. When True, it
+        expects a block, but leaves it up to the lexer to parse that block. If the
+        string "script", the block is interpreted as containing one or more
+        Ren'Py script language statements.
+
     `parse`
         This is a function that takes a Lexer object. This function should parse the
         statement, and return an object. This object is passed as an argument to all the
         other functions. The lexer argument has the following methods:
 
     `lint`
-        This is called to check the statement. It is passed a single object, the
-        argument returned from parse. It should call renpy.error to report errors.
+        This is called to check the statement. It is passed a single argument, the
+        object returned from parse. It should call renpy.error to report errors.
 
     `execute`
         This is a function that is called when the statement executes. It is passed a
-        single object, the argument returned from parse.
+        single argument, the object returned from parse.
+
+    `execute_init`
+        This is a function that is called at init time, at priority 0.
 
     `predict`
         This is a function that is called to predict the images used by the statement.
-        It is passed a single object, the argument returned from parse. It should return
+        It is passed a single argument, the object returned from parse. It should return
         a list of displayables used by the statement.
 
     `next`
-        This is called to determine the next statement. It is passed a single object,
-        the argument returned from parse. It should either return a label, or return
-        None if execution should continue to the next statement.
+        This is a function that is called to determine the next statement.
+
+        If `block` is not "script", this is passed a single argument, the object
+        returned from the parse function. If `block` is "script", an additional
+        argument is passed, an object that names the first statement in the block.
+
+        The function should return either a string giving a label to jump to,
+        the second argument to transfer control into the block, or None to
+        continue to the statement after this one.
 
     `scry`
         Used internally by Ren'Py.
 
-    `block`
-        True if this takes a block, false otherwise.
-
     `init`
         True if this statement should be run at init-time. (If the statement
         is not already inside an init block, it's automatically placed inside
-        an init 0 block.)
+        an init 0 block.) This calls the execute function, in addition to the
+        execute_init function.
     """
     name = tuple(name.split())
 
     registry[name] = dict(parse=parse,
                           lint=lint,
                           execute=execute,
+                          execute_init=execute_init,
                           predict=predict,
                           next=next,
                           scry=scry)
@@ -96,6 +110,10 @@ def register(name, parse=None, lint=None, execute=None, predict=None, next=None,
             if not block:
                 l.expect_noblock(" ".join(name) + " statement")
                 l.advance()
+            elif block == "script":
+                l.expect_block(" ".join(name) + " statement")
+                rv.code_block = renpy.parser.parse_block(l.subblock_lexer())
+                l.advance()
             else:
                 l.expect_block(" ".join(name) + " statement")
                 l.advance()
