@@ -1,4 +1,4 @@
-# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -541,6 +541,7 @@ def register_bmfont(name=None, size=None, bold=False, italics=False, underline=F
     bmf = BMFont(filename)
     image_fonts[(name, size, bold, italics)] = bmf
 
+
 # A map from face name to ftfont.FTFace
 face_cache = { }
 
@@ -565,24 +566,26 @@ def load_face(fn):
         font_file = renpy.loader.load(fn)
     except IOError:
 
-        # Let's try to find the font on our own.
-        fonts = [ i.strip().lower() for i in fn.split(",") ]
+        if (not renpy.config.developer) or renpy.config.allow_sysfonts:
 
-        pygame.sysfont.initsysfonts()
+            # Let's try to find the font on our own.
+            fonts = [ i.strip().lower() for i in fn.split(",") ]
 
-        for v in pygame.sysfont.Sysfonts.itervalues():
-            if v is not None:
-                for _flags, ffn in v.iteritems():
-                    for i in fonts:
-                        if ffn.lower().endswith(i):
-                            font_file = file(ffn, "rb")
+            pygame.sysfont.initsysfonts()
+
+            for v in pygame.sysfont.Sysfonts.itervalues():
+                if v is not None:
+                    for _flags, ffn in v.iteritems():
+                        for i in fonts:
+                            if ffn.lower().endswith(i):
+                                font_file = file(ffn, "rb")
+                                break
+
+                        if font_file:
                             break
 
-                    if font_file:
-                        break
-
-            if font_file:
-                break
+                if font_file:
+                    break
 
     if font_file is None:
         raise Exception("Could not find font {0!r}.".format(orig_fn))
@@ -593,6 +596,7 @@ def load_face(fn):
 
     return rv
 
+
 # Caches of fonts.
 image_fonts = { }
 
@@ -658,13 +662,15 @@ def free_memory():
 
     scaled_image_fonts.clear()
     font_cache.clear()
-    face_cache.clear()
 
 
-def load_image_fonts():
+def load_fonts():
     for i in image_fonts.itervalues():
         i.load()
 
+    for i in renpy.config.preload_fonts:
+        load_face(i)
+
 
 class FontGroup(object):
     """
